@@ -30,7 +30,7 @@
 //! # use gstreamer as gst;
 //! # use gst::prelude::ElementExtManual;
 //! # gst::init();
-//! # let pipeline = gst::Pipeline::new(None);
+//! # let pipeline = gst::Pipeline::new();
 //! let res = pipeline.query_position::<gst::ClockTime>();
 //! ```
 //!
@@ -40,7 +40,7 @@
 //! # use gstreamer as gst;
 //! # use gst::{format::prelude::*, prelude::ElementExtManual};
 //! # gst::init();
-//! # let pipeline = gst::Pipeline::new(None);
+//! # let pipeline = gst::Pipeline::new();
 //! # let seek_flags = gst::SeekFlags::FLUSH | gst::SeekFlags::KEY_UNIT;
 //! let seek_pos = gst::ClockTime::from_seconds(10);
 //! let res = pipeline.seek_simple(seek_flags, seek_pos);
@@ -110,11 +110,14 @@
 //! assert_eq!(*(8.mebibytes()), 8 * 1024 * 1024);
 //! assert_eq!(*(4.gibibytes()), 4 * 1024 * 1024 * 1024);
 //!
-//! // ... and the macthing constants:
+//! // ... and the matching constants:
 //! assert_eq!(512 * Bytes::KiB, 512.kibibytes());
 //!
 //! // `Percent` can be built from a percent integer value:
 //! let a_quarter = 25.percent();
+//! assert_eq!(a_quarter.percent(), 25);
+//! assert_eq!(a_quarter.ppm(), 250000);
+//! assert_eq!(a_quarter.ratio(), 0.25);
 //! // ... from a floating point ratio:
 //! let a_quarter_from_ratio = 0.25.percent_ratio();
 //! assert_eq!(a_quarter, a_quarter_from_ratio);
@@ -594,6 +597,7 @@ pub trait FormattedValueNoneBuilder: FormattedValueFullRange {
     /// Panics if `None` can't be represented by `Self` for `format` or by the requested
     /// `GenericFormattedValue` variant.
     #[track_caller]
+    #[inline]
     fn none_for_format(format: Format) -> Self {
         skip_assert_initialized!();
         // This is the default impl. `GenericFormattedValue` must override.
@@ -618,7 +622,7 @@ impl fmt::Display for Format {
             Self::Time => f.write_str("time"),
             Self::Buffers => f.write_str("buffers"),
             Self::Percent => f.write_str("%"),
-            Self::__Unknown(format) => write!(f, "(format: {})", format),
+            Self::__Unknown(format) => write!(f, "(format: {format})"),
         }
     }
 }
