@@ -6,6 +6,7 @@ use super::{
     Format, FormattedValue, FormattedValueError, FormattedValueFullRange, FormattedValueIntrinsic,
     FormattedValueNoneBuilder, GenericFormattedValue,
 };
+use crate::ffi;
 
 pub trait SpecificFormattedValue: FormattedValue {}
 
@@ -67,6 +68,14 @@ impl_format_value_traits!(Buffers, Buffers, Buffers, u64);
 option_glib_newtype_from_to!(Buffers, Buffers::OFFSET_NONE);
 glib_newtype_display!(Buffers, DisplayableOptionBuffers, Format::Buffers);
 
+impl TryFrom<Buffers> for usize {
+    type Error = std::num::TryFromIntError;
+
+    fn try_from(value: Buffers) -> Result<Self, Self::Error> {
+        value.0.try_into()
+    }
+}
+
 // FIXME `functions in traits cannot be const` (rustc 1.64.0)
 // rustdoc-stripper-ignore-next
 /// `Buffers` formatted value constructor trait.
@@ -143,6 +152,14 @@ impl_format_value_traits!(Bytes, Bytes, Bytes, u64);
 option_glib_newtype_from_to!(Bytes, u64::MAX);
 glib_newtype_display!(Bytes, DisplayableOptionBytes, Format::Bytes);
 
+impl TryFrom<Bytes> for usize {
+    type Error = std::num::TryFromIntError;
+
+    fn try_from(value: Bytes) -> Result<Self, Self::Error> {
+        value.0.try_into()
+    }
+}
+
 // FIXME `functions in traits cannot be const` (rustc 1.64.0)
 // rustdoc-stripper-ignore-next
 /// `Bytes` formatted value constructor trait.
@@ -240,6 +257,14 @@ impl_format_value_traits!(Default, Default, Default, u64);
 option_glib_newtype_from_to!(Default, u64::MAX);
 glib_newtype_display!(Default, DisplayableOptionDefault, Format::Default);
 
+impl TryFrom<Default> for usize {
+    type Error = std::num::TryFromIntError;
+
+    fn try_from(value: Default) -> Result<Self, Self::Error> {
+        value.0.try_into()
+    }
+}
+
 // FIXME `functions in traits cannot be const` (rustc 1.64.0)
 // rustdoc-stripper-ignore-next
 /// `Default` formatted value constructor trait.
@@ -311,6 +336,30 @@ impl Percent {
         // FIXME floating point arithmetic is not allowed in constant functions (rustc 1.64.0)
         Percent::try_from(ratio).expect("`Percent` ratio out of range")
     }
+
+    // rustdoc-stripper-ignore-next
+    /// The percent value in the range [0, 100].
+    #[track_caller]
+    #[inline]
+    pub fn percent(&self) -> u32 {
+        self.0 / ffi::GST_FORMAT_PERCENT_SCALE as u32
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// The per million value in the range [0, 100000].
+    #[track_caller]
+    #[inline]
+    pub fn ppm(&self) -> u32 {
+        self.0
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// The ratio value in the range [0.0, 1.0].
+    #[track_caller]
+    #[inline]
+    pub fn ratio(&self) -> f32 {
+        self.0 as f32 / ffi::GST_FORMAT_PERCENT_MAX as f32
+    }
 }
 
 impl_common_ops_for_newtype_uint!(Percent, u32, one: ffi::GST_FORMAT_PERCENT_SCALE as u32);
