@@ -12,11 +12,11 @@ use std::{
 use glib::{
     prelude::*,
     translate::*,
-    value::{FromValue, SendValue},
+    value::{FromValue, SendValue, Value},
     IntoGStr,
 };
 
-use crate::Fraction;
+use crate::{ffi, Fraction};
 
 #[derive(Clone, Debug, Eq, PartialEq, thiserror::Error)]
 pub enum GetError<E: std::error::Error> {
@@ -493,12 +493,84 @@ impl StructureRef {
         }
     }
 
+    // rustdoc-stripper-ignore-next
+    /// Sets field `name` to the given value `value`.
+    ///
+    /// Overrides any default or previously defined value for `name`.
     #[doc(alias = "gst_structure_set")]
     pub fn set(&mut self, name: impl IntoGStr, value: impl Into<glib::Value> + Send) {
         let value = glib::SendValue::from_owned(value);
         self.set_value(name, value);
     }
 
+    // rustdoc-stripper-ignore-next
+    /// Sets field `name` to the given inner value if `value` is `Some`.
+    ///
+    /// This has no effect if the `predicate` evaluates to `false`,
+    /// i.e. default or previous value for `name` is kept.
+    #[doc(alias = "gst_structure_set")]
+    pub fn set_if(
+        &mut self,
+        name: impl IntoGStr,
+        value: impl Into<glib::Value> + Send,
+        predicate: bool,
+    ) {
+        if predicate {
+            self.set(name, value);
+        }
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Sets field `name` to the given inner value if `value` is `Some`.
+    ///
+    /// This has no effect if the value is `None`, i.e. default or previous value for `name` is kept.
+    #[doc(alias = "gst_structure_set")]
+    pub fn set_if_some(
+        &mut self,
+        name: impl IntoGStr,
+        value: Option<impl Into<glib::Value> + Send>,
+    ) {
+        if let Some(value) = value {
+            self.set(name, value);
+        }
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Sets field `name` using the given `ValueType` `V` built from `iter`'s the `Item`s.
+    ///
+    /// Overrides any default or previously defined value for `name`.
+    #[inline]
+    pub fn set_from_iter<V: ValueType + Into<Value> + FromIterator<SendValue> + Send>(
+        &mut self,
+        name: impl IntoGStr,
+        iter: impl IntoIterator<Item = impl ToSendValue>,
+    ) {
+        let iter = iter.into_iter().map(|item| item.to_send_value());
+        self.set(name, V::from_iter(iter));
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Sets field `name` using the given `ValueType` `V` built from `iter`'s Item`s,
+    /// if `iter` is not empty.
+    ///
+    /// This has no effect if `iter` is empty, i.e. previous value for `name` is unchanged.
+    #[inline]
+    pub fn set_if_not_empty<V: ValueType + Into<Value> + FromIterator<SendValue> + Send>(
+        &mut self,
+        name: impl IntoGStr,
+        iter: impl IntoIterator<Item = impl ToSendValue>,
+    ) {
+        let mut iter = iter.into_iter().peekable();
+        if iter.peek().is_some() {
+            let iter = iter.map(|item| item.to_send_value());
+            self.set(name, V::from_iter(iter));
+        }
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Sets field `name` to the given value `value`.
+    ///
+    /// Overrides any default or previously defined value for `name`.
     #[doc(alias = "gst_structure_set_value")]
     pub fn set_value(&mut self, name: impl IntoGStr, value: SendValue) {
         unsafe {
@@ -508,12 +580,46 @@ impl StructureRef {
         }
     }
 
+    // rustdoc-stripper-ignore-next
+    /// Sets field `name` to the given inner value if `value` is `Some`.
+    ///
+    /// This has no effect if the `predicate` evaluates to `false`,
+    /// i.e. default or previous value for `name` is kept.
+    #[doc(alias = "gst_structure_set_value")]
+    pub fn set_value_if(&mut self, name: impl IntoGStr, value: SendValue, predicate: bool) {
+        if predicate {
+            self.set_value(name, value);
+        }
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Sets field `name` to the given inner value if `value` is `Some`.
+    ///
+    /// This has no effect if the value is `None`, i.e. default or previous value for `name` is kept.
+    #[doc(alias = "gst_structure_set_value")]
+    pub fn set_value_if_some(&mut self, name: impl IntoGStr, value: Option<SendValue>) {
+        if let Some(value) = value {
+            self.set_value(name, value);
+        }
+    }
+
     #[doc(alias = "gst_structure_id_set")]
     pub fn set_by_quark(&mut self, name: glib::Quark, value: impl Into<glib::Value> + Send) {
         let value = glib::SendValue::from_owned(value);
         self.set_value_by_quark(name, value);
     }
 
+    #[doc(alias = "gst_structure_id_set")]
+    pub fn set_by_quark_if_some(
+        &mut self,
+        name: glib::Quark,
+        value: Option<impl Into<glib::Value> + Send>,
+    ) {
+        if let Some(value) = value {
+            self.set_by_quark(name, value);
+        }
+    }
+
     #[doc(alias = "gst_structure_id_set_value")]
     pub fn set_value_by_quark(&mut self, name: glib::Quark, value: SendValue) {
         unsafe {
@@ -521,6 +627,13 @@ impl StructureRef {
         }
     }
 
+    #[doc(alias = "gst_structure_id_set_value")]
+    pub fn set_value_by_quark_if_some(&mut self, name: glib::Quark, value: Option<SendValue>) {
+        if let Some(value) = value {
+            self.set_value_by_quark(name, value);
+        }
+    }
+
     #[doc(alias = "get_name")]
     #[doc(alias = "gst_structure_get_name")]
     pub fn name<'a>(&self) -> &'a glib::GStr {
@@ -539,6 +652,13 @@ impl StructureRef {
         }
     }
 
+    #[doc(alias = "gst_structure_set_name")]
+    pub fn set_name_if_some(&mut self, name: Option<impl IntoGStr>) {
+        if let Some(name) = name {
+            self.set_name(name);
+        }
+    }
+
     #[doc(alias = "gst_structure_has_name")]
     pub fn has_name(&self, name: &str) -> bool {
         self.name() == name
@@ -615,13 +735,13 @@ impl StructureRef {
 
     #[doc(alias = "get_nth_field_name")]
     #[doc(alias = "gst_structure_nth_field_name")]
-    pub fn nth_field_name<'a>(&self, idx: u32) -> Option<&'a glib::GStr> {
+    pub fn nth_field_name<'a>(&self, idx: usize) -> Option<&'a glib::GStr> {
         if idx >= self.n_fields() {
             return None;
         }
 
         unsafe {
-            let field_name = ffi::gst_structure_nth_field_name(&self.0, idx);
+            let field_name = ffi::gst_structure_nth_field_name(&self.0, idx as u32);
             debug_assert!(!field_name.is_null());
 
             Some(glib::GStr::from_ptr(field_name))
@@ -629,8 +749,16 @@ impl StructureRef {
     }
 
     #[doc(alias = "gst_structure_n_fields")]
-    pub fn n_fields(&self) -> u32 {
-        unsafe { ffi::gst_structure_n_fields(&self.0) as u32 }
+    pub fn n_fields(&self) -> usize {
+        unsafe { ffi::gst_structure_n_fields(&self.0) as usize }
+    }
+
+    pub fn len(&self) -> usize {
+        self.n_fields()
+    }
+
+    pub fn is_empty(&self) -> bool {
+        self.n_fields() == 0
     }
 
     #[doc(alias = "gst_structure_can_intersect")]
@@ -958,7 +1086,7 @@ impl<'a> FieldIterator<'a> {
         FieldIterator {
             structure,
             idx: 0,
-            n_fields: n_fields as usize,
+            n_fields,
         }
     }
 }
@@ -971,7 +1099,7 @@ impl<'a> Iterator for FieldIterator<'a> {
             return None;
         }
 
-        let field_name = self.structure.nth_field_name(self.idx as u32).unwrap();
+        let field_name = self.structure.nth_field_name(self.idx).unwrap();
         self.idx += 1;
 
         Some(field_name)
@@ -991,7 +1119,7 @@ impl<'a> DoubleEndedIterator for FieldIterator<'a> {
         }
 
         self.n_fields -= 1;
-        Some(self.structure.nth_field_name(self.n_fields as u32).unwrap())
+        Some(self.structure.nth_field_name(self.n_fields).unwrap())
     }
 }
 
@@ -1116,11 +1244,18 @@ impl Builder {
         }
     }
 
+    // rustdoc-stripper-ignore-next
+    /// Sets field `name` to the given value `value`.
+    ///
+    /// Overrides any default or previously defined value for `name`.
+    #[inline]
     pub fn field(mut self, name: impl IntoGStr, value: impl Into<glib::Value> + Send) -> Self {
         self.s.set(name, value);
         self
     }
 
+    impl_builder_gvalue_extra_setters!(field);
+
     #[must_use = "Building the structure without using it has no effect"]
     pub fn build(self) -> Structure {
         self.s
@@ -1141,8 +1276,10 @@ mod tests {
         assert_eq!(s.name(), "test");
 
         s.set("f1", "abc");
-        s.set("f2", &String::from("bcd"));
+        s.set("f2", String::from("bcd"));
         s.set("f3", 123i32);
+        s.set("f5", Some("efg"));
+        s.set("f7", 42i32);
 
         assert_eq!(s.get::<&str>("f1"), Ok("abc"));
         assert_eq!(s.get::<Option<&str>>("f2"), Ok(Some("bcd")));
@@ -1151,6 +1288,8 @@ mod tests {
         assert_eq!(s.get_optional::<&str>("f4"), Ok(None));
         assert_eq!(s.get_optional::<i32>("f3"), Ok(Some(123i32)));
         assert_eq!(s.get_optional::<i32>("f4"), Ok(None));
+        assert_eq!(s.get::<&str>("f5"), Ok("efg"));
+        assert_eq!(s.get::<i32>("f7"), Ok(42i32));
 
         assert_eq!(
             s.get::<i32>("f2"),
@@ -1172,23 +1311,47 @@ mod tests {
         );
         assert_eq!(s.get::<i32>("f4"), Err(GetError::new_field_not_found("f4")));
 
-        assert_eq!(s.fields().collect::<Vec<_>>(), vec!["f1", "f2", "f3"]);
+        assert_eq!(
+            s.fields().collect::<Vec<_>>(),
+            vec!["f1", "f2", "f3", "f5", "f7"]
+        );
 
         let v = s.iter().map(|(f, v)| (f, v.clone())).collect::<Vec<_>>();
-        assert_eq!(v.len(), 3);
+        assert_eq!(v.len(), 5);
         assert_eq!(v[0].0, "f1");
         assert_eq!(v[0].1.get::<&str>(), Ok("abc"));
         assert_eq!(v[1].0, "f2");
         assert_eq!(v[1].1.get::<&str>(), Ok("bcd"));
         assert_eq!(v[2].0, "f3");
         assert_eq!(v[2].1.get::<i32>(), Ok(123i32));
+        assert_eq!(v[3].0, "f5");
+        assert_eq!(v[3].1.get::<&str>(), Ok("efg"));
+        assert_eq!(v[4].0, "f7");
+        assert_eq!(v[4].1.get::<i32>(), Ok(42i32));
 
         let s2 = Structure::builder("test")
             .field("f1", "abc")
             .field("f2", String::from("bcd"))
             .field("f3", 123i32)
+            .field_if_some("f4", Option::<i32>::None)
+            .field_if_some("f5", Some("efg"))
+            .field_if_some("f6", Option::<&str>::None)
+            .field_if("f7", 42i32, true)
+            .field_if("f8", 21i32, false)
             .build();
         assert_eq!(s, s2);
+
+        let mut s3 = Structure::new_empty("test");
+
+        s3.set_if_some("f1", Some("abc"));
+        s3.set_if_some("f2", Some(String::from("bcd")));
+        s3.set_if_some("f3", Some(123i32));
+        s3.set_if_some("f4", Option::<i32>::None);
+        s3.set_if_some("f5", Some("efg"));
+        s3.set_if_some("f6", Option::<&str>::None);
+        s3.set_if("f7", 42i32, true);
+        s3.set_if("f8", 21i32, false);
+        assert_eq!(s, s3);
     }
 
     #[test]
@@ -1257,4 +1420,64 @@ mod tests {
 
         assert_eq!(format!("{s:?}"), "Structure(test { f1: (gchararray) \"abc\", f2: (gchararray) \"bcd\", f3: (gint) 123, f4: Structure(nested { badger: (gboolean) TRUE }), f5: Array([(gchararray) \"a\", (gchararray) \"b\", (gchararray) \"c\"]), f6: List([(gchararray) \"d\", (gchararray) \"e\", (gchararray) \"f\"]) })");
     }
+
+    #[test]
+    fn builder_field_from_iter() {
+        crate::init().unwrap();
+
+        let s = Structure::builder("test")
+            .field_from_iter::<crate::Array>("array", [&1, &2, &3])
+            .field_from_iter::<crate::List>("list", [&4, &5, &6])
+            .build();
+        assert!(s
+            .get::<crate::Array>("array")
+            .unwrap()
+            .iter()
+            .map(|val| val.get::<i32>().unwrap())
+            .eq([1, 2, 3]));
+        assert!(s
+            .get::<crate::List>("list")
+            .unwrap()
+            .iter()
+            .map(|val| val.get::<i32>().unwrap())
+            .eq([4, 5, 6]));
+
+        let array = Vec::<i32>::new();
+        let s = Structure::builder("test")
+            .field_from_iter::<crate::Array>("array", &array)
+            .field_from_iter::<crate::List>("list", &array)
+            .build();
+        assert!(s.get::<crate::Array>("array").unwrap().as_ref().is_empty());
+        assert!(s.get::<crate::List>("list").unwrap().as_ref().is_empty());
+    }
+
+    #[test]
+    fn builder_field_if_not_empty() {
+        crate::init().unwrap();
+
+        let s = Structure::builder("test")
+            .field_if_not_empty::<crate::Array>("array", [&1, &2, &3])
+            .field_if_not_empty::<crate::List>("list", [&4, &5, &6])
+            .build();
+        assert!(s
+            .get::<crate::Array>("array")
+            .unwrap()
+            .iter()
+            .map(|val| val.get::<i32>().unwrap())
+            .eq([1, 2, 3]));
+        assert!(s
+            .get::<crate::List>("list")
+            .unwrap()
+            .iter()
+            .map(|val| val.get::<i32>().unwrap())
+            .eq([4, 5, 6]));
+
+        let array = Vec::<i32>::new();
+        let s = Structure::builder("test")
+            .field_if_not_empty::<crate::Array>("array", &array)
+            .field_if_not_empty::<crate::List>("list", &array)
+            .build();
+        assert!(!s.has_field("array"));
+        assert!(!s.has_field("list"));
+    }
 }
